﻿//---------------------------------------------------------------------
// <copyright file="IUriLiteralParser.cs" company="Microsoft">
//      Copyright (C) Microsoft Corporation. All rights reserved. See License.txt in the project root for license information.
// </copyright>
//---------------------------------------------------------------------

using Microsoft.OData.Edm;

namespace Microsoft.OData.UriParser
{
    /// <summary>
    /// Interface for Uri literal parser.
    /// To parse the uri of an OData request into objects, the ODataUriParser uses UriLiteralParses.
    /// Implementation of this interface will parse a text of an EdmType to it's instance.
    /// </summary>
    public interface IUriLiteralParser
    {
        /// <summary>
        /// Parse the given text of EdmType <paramref name="targetType"/> to it's object instance.
        /// Return 'Null' if the text could not be parsed to the requested <paramref name="targetType"/>.
        /// Assign <paramref name="parsingException"/> parameter only in case the text could be parsed to the requested <paramref name="targetType"/>, but failed during the parsing process.
        /// </summary>
        /// <param name="text">Part of the uri which has to be parsed to a value of EdmType <paramref name="targetType"/></param>
        /// <param name="targetType">The type which the uri text has to be parsed to</param>
        /// <param name="parsingException">Assign the exception only in case the text could be parsed to the <paramref name="targetType"/> but failed during the parsing process</param>
        /// <returns>If the parsing process has succeeded, returns the parsed object, otherwise returns 'Null'</returns>
        object ParseUriStringToType(string text, IEdmTypeReference targetType, out UriLiteralParsingException parsingException);

        // Consider add this API:
        // bool TryParseUriStringToType(string text, IEdmTypeReference targetType,out object targetValue, out UriTypeParsingException parsingException);
        //      This can be problematic because of the the 'bool' return type + the out exception parameter. The 'Try' function could return 'false' with null exception because it doesn't support the given type,
        //      bnd this not a 'standard' function so it can confuse the developers. Standard 'Try' function assign the out Exception parameter when return value is 'false'.
    }
}
